/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

This file is part of Scotty.

Scotty is free software: you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later version.

Scotty is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
Scotty. If not, see <https://www.gnu.org/licenses/>.
*/

package backends

import (
	"net/url"
	"strings"

	"github.com/spf13/viper"
	"go.uploadedlobster.com/scotty/models"
	"golang.org/x/oauth2"
)

func BuildRedirectURL(config *viper.Viper, backend string) (*url.URL, error) {
	callbackHost, _ := strings.CutSuffix(config.GetString("oauth-host"), "/")
	if callbackHost == "" {
		callbackHost = "127.0.0.1:2369"
	}
	callbackPath := "/callback/" + backend
	return url.Parse("http://" + callbackHost + callbackPath)
}

func Authenticate(backend models.Backend, token *oauth2.Token, config *viper.Viper) (bool, error) {
	authenticator, auth := backend.(models.OAuth2Authenticator)
	if auth {
		// FIXME
		redirectURL, err := BuildRedirectURL(config, backend.Name())
		if err != nil {
			return auth, err
		}
		authenticator.OAuth2Setup(redirectURL, token)
	}
	return auth, nil
}
